/**************************************************
*	Calculates PCA eccentricity
*	For details see http://arxiv.org/abs/1002.3982
**************************************************/

using namespace std; 
#include<iostream>
#include "CParticle.h"
#include "LParticle.h"
#include "TMath.h"
#include "TLorentzVector.h"


// see implementation details in
// http://ask.metafilter.com/36213/Best-Fit-Ellipse


double eccentricity(LParticle jet) {

   vector<CParticle> constit = jet.GetConstituents();
   int num=constit.size();
   TLorentzVector TJet = jet.GetP();
 
   float Dphi[num],Deta[num];  
   float etaSum = 0.; float phiSum = 0.; float eTot = 0.; 

   for (int j=0; j< num; j++) {
	CParticle cp = constit.at(j);
	TLorentzVector tv(cp.px(),cp.py(),cp.pz(),cp.e());
        Dphi[j] = TJet.Phi() - tv.Phi();
        if (Dphi[j]>TMath::Pi()) Dphi[j]=2*TMath::Pi()-Dphi[j]; 
        Deta[j] = TJet.PseudoRapidity() - tv.PseudoRapidity();
        etaSum = etaSum + Deta[j] * cp.e();
        phiSum = phiSum + Dphi[j] * cp.e();
        eTot   = eTot + cp.e();
   } 

  etaSum = etaSum/eTot; phiSum = phiSum/eTot;
  for(unsigned int j = 0; j< num; j++) {
    Deta[j] = Deta[j]-etaSum;
    Dphi[j] = Dphi[j]-phiSum;
   }


   float X1 = 0.; float X2=0; 
   for(int i = 0; i < num; i++) {
    CParticle cp = constit.at(i);
    X1 += 2. * cp.e() * Deta[i] * Dphi[i];
    X2 += cp.e() * ( Dphi[i] * Dphi[i] - Deta[i] * Deta[i] );
   }


   // variance calculations 
   float Theta = .5 * atan( X1/X2 );
   float sinTheta = TMath::Sin(Theta);
   float cosTheta = TMath::Cos(Theta);
   float Theta2 = Theta + 0.5*TMath::Pi();
   float sinThetaPrime = TMath::Sin(Theta2);
   float cosThetaPrime = TMath::Cos(Theta2);
   float VarX = 0.;
   float VarY = 0.;


   for(int i = 0; i < num; i++) {
     CParticle cp = constit.at(i);
     float X=cosTheta*Deta[i] - sinTheta*Dphi[i];    
     float Y=sinTheta*Deta[i] + cosTheta*Dphi[i]; 
     VarX += cp.e()*X*X;
     VarY += cp.e()*Y*Y;
   }


// another option (atlas?)
    float VarX1 = 0.;
    float VarY1 = 0.;
    for(unsigned int i = 0; i < num; i++) {
    CParticle cp = constit.at(i);
    double t1=2.*sinTheta*cosTheta*Deta[i]*Dphi[i];
    double t2=cosTheta*cosTheta*Dphi[i]*Dphi[i];
    double t3=sinTheta*sinTheta*Deta[i]*Deta[i];
    VarX1 += cp.e()*(t1+ t2+ t3);
    t1=2.*sinThetaPrime*cosThetaPrime*Deta[i]*Dphi[i];
    t2=cosThetaPrime*cosThetaPrime*Dphi[i]*Dphi[i];
    t3=sinThetaPrime*sinThetaPrime*Deta[i]*Deta[i];
    VarY1 += cp.e()*(t1+t2+t3);
    }



  float VarianceMax = VarX; float VarianceMin = VarY; 
  if(VarianceMax < VarianceMin) {
    VarianceMax = VarY;
    VarianceMin = VarX;
  }
   double ECC=1. - (VarianceMin/VarianceMax);
   // cout << "1st: VarXmax = " << VarianceMax << "   VarXmin=" << VarianceMin << "  ECC=" << ECC << endl;


// second option
  VarianceMax = VarX1; VarianceMin = VarY1;
  if(VarianceMax < VarianceMin) {
    VarianceMax = VarY1;
    VarianceMin = VarX1;
  }
   double ECC1=1. - (VarianceMin/VarianceMax);
   // cout << "2 nd: VarXmax = " << VarianceMax << "   VarXmin=" << VarianceMin << "  ECC=" << ECC1 << endl;



  return ECC;


} 
