/*********************************************************************************
*	calculates 'pull' & moments of covariance tensor
*	T[0] is magnitude of T, T[1] is angle between jet momentum and beam (theta)
*       T[higher] are moments of the covariance tensor
*	for details, see http://arxiv.org/abs/1001.5027
**********************************************************************************/

#include<iostream>
#include "LParticle.h"
#include "CParticle.h"
#include "TMath.h"
#include "TLorentzVector.h"
#include<vector>

double* pull(LParticle jet) {

	int i;
	double* T = new double[6];
	for (i=0; i<5; i++){T[i] = 0;}
	vector<CParticle> constit = jet.GetConstituents();
	int num = constit.size();
	double Pt;
	double r[num][2];
	TLorentzVector TJet = jet.GetP();
	double Jet_Pt = TMath::Sqrt((TJet.Px() * TJet.Px()) + (TJet.Py() * TJet.Py()));

	for (i = 0; i < num; i++){
		CParticle cp = constit.at(i);
                TLorentzVector tv(cp.px(),cp.py(),cp.pz(),cp.e());
		double pX=(double)cp.px()/1000.0;
		double pY=(double)cp.py()/1000.0;
		Pt = TMath::Sqrt((pX * pX) + (pY * pY));
		r[i][0] = tv.PseudoRapidity() - TJet.PseudoRapidity();
		r[i][1] = tv.Phi() - TJet.Phi();
		T[2] += (tv.PseudoRapidity() * tv.PseudoRapidity()) - (TJet.PseudoRapidity() * TJet.PseudoRapidity());
		T[3] += r[i][0] * r[i][1];
		T[4] += r[i][1] * r[i][0];
		T[5] += (tv.Phi() * tv.Phi()) - (TJet.Phi() * TJet.Phi());
		T[0] += (Pt * TMath::Sqrt((r[i][0]*r[i][0]) + (r[i][1]*r[i][1])))/Jet_Pt;
	}
	T[1] =( 2 * atan(TMath::Exp(-TJet.PseudoRapidity()))) / TMath::Pi();

	if (T[0] != T[0]){
		for (i = 0; i < num; i++){
			CParticle cp = constit.at(i);
			TLorentzVector tv(cp.px(),cp.py(),cp.pz(),cp.e());
			cout<<"WARNING:PULL=NAN*******************WARNING:PULL=NAN*******************************WARNING:PULL=NAN"<<endl;
		}
	}



return T;
}

