/************************************************************
*	Calculates PCA jet width
*	For details see http://arxiv.org/abs/1002.3982
************************************************************/

#include<iostream>
#include "LParticle.h"
#include "CParticle.h"
#include "TMath.h"
#include "TLorentzVector.h"
#include<vector>

double Width(LParticle jet){
        int i;
	double jetWidth, jetPtSum;
        vector<CParticle> constit = jet.GetConstituents();
	TLorentzVector TJet = jet.GetP();
        int num = constit.size();
        
	for (int j=0; j< num; j++) {
		CParticle cp = constit.at(j);
		TLorentzVector tv(cp.px(),cp.py(),cp.pz(),cp.e());
		double Pt = TMath::Sqrt((cp.px()*cp.px())+(cp.py()*cp.py()));
        	if (Pt > 0) {
        	double dPhi = TJet.Phi() - tv.Phi();
        	if (dPhi>TMath::Pi()) dPhi=2*TMath::Pi()-dPhi;
        	double dEta = TJet.PseudoRapidity() - tv.PseudoRapidity();
        	double dR=TMath::Sqrt(dPhi*dPhi + dEta *dEta);
        	jetWidth += dR * Pt;
        	jetPtSum += Pt;
        	}
   	}

   	jetWidth = jetWidth / jetPtSum;
return jetWidth;
}
